/*
 * mount.pvfs.c copyright (c) 1997 Clemson University, all rights reserved.
 *
 * Written by Rob Ross, based somewhat on nfs mount code by Rick Sladkey
 * <jrs@world.std.com>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Contact:  Rob Ross    rbross@parl.eng.clemson.edu
 */

#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <unistd.h>

#include <sys/socket.h>
#include <sys/mount.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <pvfs_mount.h> 

#define DEBUG 0
#define dprintf(args...) if (DEBUG) printf(args);

int usage(int argc, char **argv);
int parse_args(int argc, char **argv);
struct in_addr ResolveAddr(char *name);

int main(int argc, char **argv)
{
	int ret, flags = 0;
	char hostdir[2*PVFS_MGRLEN], type[] = "pvfs";
	char *hostname, *dirname, *special, *mountpoint;
	struct pvfs_mount data;
	struct in_addr haddr;

	if (argc < 2) {
		fprintf(stderr, "%s: too few arguments\n", argv[0]);
		exit(0);
	}

	/* for now assume host:dir is in argv[1] */
	/* split host:dir into components */
	if (strlen(argv[1]) > sizeof(hostdir)) {
		fprintf(stderr, "%s: excessively long host:dir argument\n", argv[0]);
		exit(0);
	}
	strncpy(hostdir, argv[1], sizeof(hostdir));
	if ((dirname = (strchr(hostdir, ':')))) {
		hostname = hostdir;
		*dirname = '\0';
		dirname++;
	}
	else /* not good */ {
		fprintf(stderr, "%s: directory to mount not in host:dir format\n",
			argv[0]);
		exit(0);
	}

	haddr = ResolveAddr(hostname);
	hostname = inet_ntoa(haddr);

	dprintf("hostname: %s, dirname: %s\n", hostname, dirname);

	data.info_magic = 0;
	data.flags   = 0;
	data.port    = 3000;
	strncpy(data.mgr, hostname, PVFS_MGRLEN);
	strncpy(data.dir,  dirname,  PVFS_DIRLEN);

	special = argv[1];
	mountpoint = argv[2];
	flags |= 0xC0ED0000;

	dprintf("calling mount(special = %s, mountpoint = %s, type = %s,"
	" flags = %x, data = %x)\n", special, mountpoint, type, flags,
	(unsigned int) &data);
	if ((ret = mount(special, mountpoint, type, flags, (void *) &data)) < 0)
	{
		perror(argv[0]);
	}
	return(ret);
} /* end of main() */

/* ResolveAddr - returns struct in_addr holding address of a host
 *               specified by the character string name, which can
 *               either be a host name or IP address in dot notation.
 */
struct in_addr ResolveAddr(char *name)
{
   struct in_addr addr;
   struct hostent *hep;

   if (!inet_aton(name, &addr)) {
      if (!(hep=gethostbyname(name))) bzero((char *)&addr, sizeof(addr));
      else bcopy(hep->h_addr, (char *)&addr, hep->h_length);
   }
   return(addr);
}


int parse_args(int argc, char **argv)
{
	int c;

	while ((c = getopt(argc, argv, "")) != EOF) {
		switch(c) {
			case '?':
				usage(argc, argv);
				break;
		}
	}
	return 0;
} /* end of PARSE_ARGS */

int usage(int argc, char **argv)
{
	fprintf(stderr, "%s: <options> <filename> ...\n", argv[0]);
	return 0;
} /* end of USAGE() */

